-------------------------------------------------------------------------------
-- Title      : toplevel entity of crossbar
-- Project    : AXI-3 Crossbar Switch
-------------------------------------------------------------------------------
-- File       : axi3-interconnect.vhd
-- Author     : Mario Hüttel <mario.huettel@gmx.net>
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2016
--
-- This file is part of AXI-3 Crossbar Switch.
--
-- AXI-3 Crossabr Switch is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation,  version 2 of the License.
--
-- This code is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this code.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
-- "work" denotes the curent library. Similar to this in C++, C# etc...
use work.axi3intercon_pkg.all;
use work.axi_aw_router_pkg.all;
use work.axi_ar_router_pkg.all;
use work.axi_r_router_pkg.all;
use work.axi_w_router_pkg.all;
use work.axi_b_router_pkg.all;

entity axi3intercon is
	port(
		aclk          : in  std_logic;
		aresetn       : in  std_logic;
		masters_in    : out axi_masters_in_t(0 to MASTER_COUNT - 1);
		masters_out   : in  axi_masters_out_t(0 to MASTER_COUNT - 1);
		slaves_in     : out axi_slaves_in_t(0 to SLAVE_COUNT - 1);
		slaves_out    : in  axi_slaves_out_t(0 to SLAVE_COUNT - 1);
		address_array : in  axi_slave_addresses_t(0 to SLAVE_COUNT - 1);
		mask_array    : in  axi_slave_addresses_t(0 to SLAVE_COUNT - 1)
	);
end entity axi3intercon;

architecture RTL of axi3intercon is
	signal rst            : std_logic;
	signal write_locks    : write_locks_t(0 to MASTER_COUNT - 1);
	signal write_releases : write_releases_t;
	signal aw_masters_out : axi_aw_masters_out_t(0 to MASTER_COUNT - 1);
	signal aw_masters_in  : axi_aw_masters_in_t(0 to MASTER_COUNT - 1);
	signal aw_slaves_out  : axi_aw_slaves_out_t(0 to SLAVE_COUNT);
	signal aw_slaves_in   : axi_aw_slaves_in_t(0 to SLAVE_COUNT);
	signal ar_masters_out : axi_ar_masters_out_t(0 to MASTER_COUNT - 1);
	signal ar_masters_in  : axi_ar_masters_in_t(0 to MASTER_COUNT - 1);
	signal ar_slaves_out  : axi_ar_slaves_out_t(0 to SLAVE_COUNT);
	signal ar_slaves_in   : axi_ar_slaves_in_t(0 to SLAVE_COUNT);
	signal w_masters_in   : axi_w_masters_in_t(0 to MASTER_COUNT - 1);
	signal w_masters_out  : axi_w_masters_out_t(0 to MASTER_COUNT - 1);
	signal w_slaves_in    : axi_w_slaves_in_t(0 to SLAVE_COUNT);
	signal w_slaves_out   : axi_w_slaves_out_t(0 to SLAVE_COUNT);
	signal r_slaves_in    : axi_r_slaves_in_t(0 to SLAVE_COUNT);
	signal r_slaves_out   : axi_r_slaves_out_t(0 to SLAVE_COUNT);
	signal r_masters_in   : axi_r_masters_in_t(0 to MASTER_COUNT - 1);
	signal r_masters_out  : axi_r_masters_out_t(0 to MASTER_COUNT - 1);
	signal b_slaves_in    : axi_b_slaves_in_t(0 to SLAVE_COUNT);
	signal b_slaves_out   : axi_b_slaves_out_t(0 to SLAVE_COUNT);
	signal b_masters_in   : axi_b_masters_in_t(0 to MASTER_COUNT - 1);
	signal b_masters_out  : axi_b_masters_out_t(0 to MASTER_COUNT - 1);

	signal decerr_in  : axi_slave_in_t;
	signal decerr_out : axi_slave_out_t;

begin
	reset_sync : process(aclk, aresetn) is
	begin
		if aresetn = '0' then
			rst <= '1';
		elsif rising_edge(aclk) then
			rst <= '0';
		end if;
	end process reset_sync;

	axi3intercon_aw_router_inst : entity work.axi3intercon_aw_router
		port map(
			aclk           => aclk,
			rst            => rst,
			masters_out    => aw_masters_out,
			masters_in     => aw_masters_in,
			slaves_out     => aw_slaves_out,
			slaves_in      => aw_slaves_in,
			write_locks    => write_locks,
			write_releases => write_releases,
			address_array  => address_array,
			mask_array     => mask_array
		);

	aw_master_connect : for i in 0 to MASTER_COUNT - 1 generate
		aw_masters_out(i) <= masters_out(i).aw;
		masters_in(i).aw  <= aw_masters_in(i);
	end generate aw_master_connect;

	aw_slave_connect : for i in 0 to SLAVE_COUNT - 1 generate
		aw_slaves_out(i) <= slaves_out(i).aw;
		slaves_in(i).aw  <= aw_slaves_in(i);
	end generate aw_slave_connect;

	axi3intercon_ar_router_inst : entity work.axi3intercon_ar_router
		port map(
			aclk          => aclk,
			rst           => rst,
			masters_out   => ar_masters_out,
			masters_in    => ar_masters_in,
			slaves_out    => ar_slaves_out,
			slaves_in     => ar_slaves_in,
			address_array => address_array,
			mask_array    => mask_array
		);

	ar_master_connect : for i in 0 to MASTER_COUNT - 1 generate
		ar_masters_out(i) <= masters_out(i).ar;
		masters_in(i).ar  <= ar_masters_in(i);
	end generate ar_master_connect;

	ar_slave_connect : for i in 0 to SLAVE_COUNT - 1 generate
		ar_slaves_out(i) <= slaves_out(i).ar;
		slaves_in(i).ar  <= ar_slaves_in(i);
	end generate ar_slave_connect;

	axi3_intercon_w_router_inst : entity work.axi3_intercon_w_router
		port map(
			clk            => aclk,
			rst            => rst,
			masters_in     => w_masters_in,
			masters_out    => w_masters_out,
			slaves_in      => w_slaves_in,
			slaves_out     => w_slaves_out,
			write_locks    => write_locks,
			write_releases => write_releases
		);

	w_master_connect : for i in 0 to MASTER_COUNT - 1 generate
		w_masters_out(i) <= masters_out(i).w;
		masters_in(i).w  <= w_masters_in(i);
	end generate w_master_connect;

	w_slave_connect : for i in 0 to SLAVE_COUNT - 1 generate
		w_slaves_out(i) <= slaves_out(i).w;
		slaves_in(i).w  <= w_slaves_in(i);
	end generate w_slave_connect;

	axi3_intercon_r_router_inst : entity work.axi3_intercon_r_router
		port map(
			clk         => aclk,
			rst         => rst,
			slaves_in   => r_slaves_in,
			slaves_out  => r_slaves_out,
			masters_in  => r_masters_in,
			masters_out => r_masters_out
		);

	r_master_connect : for i in 0 to MASTER_COUNT - 1 generate
		r_masters_out(i) <= masters_out(i).r;
		masters_in(i).r  <= r_masters_in(i);
	end generate r_master_connect;

	r_slave_connect : for i in 0 to SLAVE_COUNT - 1 generate
		r_slaves_out(i) <= slaves_out(i).r;
		slaves_in(i).r  <= r_slaves_in(i);
	end generate r_slave_connect;

	axi3_intercon_b_router_inst : entity work.axi3_intercon_b_router
		port map(
			clk         => aclk,
			rst         => rst,
			slaves_in   => b_slaves_in,
			slaves_out  => b_slaves_out,
			masters_in  => b_masters_in,
			masters_out => b_masters_out
		);

	b_master_connect : for i in 0 to MASTER_COUNT - 1 generate
		b_masters_out(i) <= masters_out(i).b;
		masters_in(i).b  <= b_masters_in(i);
	end generate b_master_connect;

	b_slave_connect : for i in 0 to SLAVE_COUNT - 1 generate
		b_slaves_out(i) <= slaves_out(i).b;
		slaves_in(i).b  <= b_slaves_in(i);
	end generate b_slave_connect;

	-- ERROR slave connections

	axi3decerr_inst : entity work.axi3decerr
		port map(
			clk       => aclk,
			rst       => rst,
			slave_in  => decerr_in,
			slave_out => decerr_out
		);

	decerr_in.ar <= ar_slaves_in(SLAVE_COUNT);
	decerr_in.aw <= aw_slaves_in(SLAVE_COUNT);
	decerr_in.r  <= r_slaves_in(SLAVE_COUNT);
	decerr_in.w  <= w_slaves_in(SLAVE_COUNT);
	decerr_in.b  <= b_slaves_in(SLAVE_COUNT);

	ar_slaves_out(SLAVE_COUNT) <= decerr_out.ar;
	aw_slaves_out(SLAVE_COUNT) <= decerr_out.aw;
	r_slaves_out(SLAVE_COUNT)  <= decerr_out.r;
	w_slaves_out(SLAVE_COUNT)  <= decerr_out.w;
	b_slaves_out(SLAVE_COUNT)  <= decerr_out.b;

end architecture RTL;
